import os
import platform
import sys
import tempfile
import tarfile

base = os.getcwd()

PYTHON_VERSION = ".".join((str(sys.version_info[0]), str(sys.version_info[1])))
OS_VERSION = tuple(int(v) for v in platform.mac_ver()[0].split('.'))

# distribution version
DISTRIBUTION_VERSION = 1

# dependencies name and version
bzr = {'name':'bzr',
       'version':'2.1.0b1',
       'destdir':'bzr',
       'url':'http://launchpad.net/bzr/2.1/2.1.0b1/+download/bzr-2.1.0b1.tar.gz'}
bzrtools = {'name':'bzrtools',
            'version':'2.0.0',
            'destdir':'bzrtools',
            'url':'http://launchpad.net/bzrtools/stable/2.0.0/+download/bzrtools-2.0.0.tar.gz'}
email = {'name':'bzr-email',
         'revno':'40',
         'destdir':'email',
         'branch':'lp:bzr-email'}
extmerge = {'name':'bzr-extmerge',
            'revno':'13',
            'destdir':'extmerge',
            'branch':'lp:bzr-extmerge'}
loom = {'name':'bzr-loom',
        'version':'1.4.0dev0',
        'revno':'101',
        'destdir':'loom',
        'branch':'lp:bzr-loom'}
paramiko = {'name':'paramiko',
            'version':'1.7.4',
            'destdir':'paramiko',
            'url':'http://www.lag.net/paramiko/download/paramiko-1.7.4.tar.gz'}
pycrypto = {'name':'pycrypto',
            'version':'2.0.1',
            'destdir':'pycrypto',
            'url':'http://www.amk.ca/files/python/crypto/pycrypto-2.0.1.tar.gz',}
rewrite = {'name':'bzr-rewrite',
           'version':'0.5.4',
           'destdir':'rebase',
           'url':'http://samba.org/~jelmer/bzr/bzr-rewrite-0.5.4.tar.gz'}
search = {'name':'bzr-search',
          'revno':'72',
          'destdir':'search',
          'branch':'lp:bzr-search'}
subvertpy = {'name':'subvertpy',
             'version':'0.6.9',
             'destdir':'subvertpy',
             'url':'http://edge.launchpad.net/subvertpy/trunk/0.6.9/+download/subvertpy-0.6.9.tar.gz'}
svn = {'name':'bzr-svn', 
       'version':'1.0.0', 
       'destdir':'svn',
       'url':'http://samba.org/~jelmer/bzr/bzr-svn-1.0.0.tar.gz'}
upload = {'name':'bzr-upload',
          'revno':'66',
          'destdir':'upload',
          'branch':'lp:bzr-upload'}
xmloutput = {'name':'bzr-xmloutput',
        'version':'0.8.6dev',
        'revno':'136',
        'destdir':'xmloutput',
        'branch':'lp:bzr-xmloutput'}
keychain = {'name':'bzr-keychain',
            'revno':'15',
            'destdir':'keychain',
            'branch':'lp:bzr-keychain'}

# UI stuff
pyqt = {'name':'PyQt-mac-gpl',
        'version':'4.6',
        'destdir':'pyqt',
        'url':'http://www.riverbankcomputing.co.uk/static/Downloads/PyQt4/PyQt-mac-gpl-4.6.tar.gz'}
sip = {'name':'sip',
        'version':'4.9',
        'destdir':'sip',
        'url':'http://www.riverbankcomputing.co.uk/static/Downloads/sip4/sip-4.9.tar.gz'}        
qbzr = {'name':'qbzr',
        'version':'0.14.3',
        'destdir':'qbzr',
        'url':'http://launchpad.net/qbzr/0.14/0.14.3/+download/qbzr-0.14.3.tar.gz'}
explorer = {'name':'explorer',
        'version':'0.8.2',
        'destdir':'explorer',
        'branch':'lp:bzr-explorer'}

## end of configurable stuff ##
packages = {'paramiko':paramiko,
            'pycrypto':pycrypto,
            'bzr':bzr,}
plugins = {'email':email,
           'rewrite':rewrite,
           'bzrtools':bzrtools,
           'search':search,
           'upload':upload,
           'loom':loom,
           'xmloutput':xmloutput}
desktop_packages = {'sip': sip, 'pyqt': pyqt}
desktop_plugins = {'extmerge': extmerge, 'qbzr': qbzr}

if OS_VERSION[1] >= 5:
    plugins['svn'] = svn
    plugins['keychain'] = keychain
    packages['subvertpy'] = subvertpy

# utilities
def extract(src, dest):
    tmpdir = tempfile.mkdtemp()
    tar = tarfile.open(src)
    tar.extractall(tmpdir)
    tar.close()
    tmp = os.listdir(tmpdir)[0]
    os.rename(os.path.join(tmpdir, tmp), os.path.join(tmpdir, dest))
    return os.path.join(tmpdir, dest)
    
def get_tarfile(plugin):
    if plugin.get('version', None) is None:
        return plugin['name'] + '-r' + plugin['revno'] + '.tar.gz'
    else:
        return plugin['name']+'-'+plugin['version'] + '.tar.gz'

