#!/usr/bin/env python
from __future__ import with_statement
import os
import subprocess
import sys
import urllib, urllib2
from optparse import OptionParser

from bzrlib import branch


def check_revno(tree_dir, dep):
    b = branch.Branch.open_containing(tree_dir)[0]
    b.lock_read()
    try:
        revno = b.revno()
    finally:
        b.unlock()
    match = str(revno).strip() == dep['revno'].strip()
    if not match:
        print >>sys.stderr, "Warning: revno mismatch in %s, expected: %r but was: %r" % (dep['name'], dep['revno'], str(revno))
    return match


class ProgressSpinner(object):
    """Un spinner *muy* simple para mostrar actividad en la terminal"""

    def __init__(self, out=sys.stderr):
        """inicializa el spinner"""
        self.out = out
        self.running = True
        self.out.write('\n')
        def spinner_generator():
            while self.running:
                for i in ('|', '/', '-', '\\', '|', '/', '-', '\\'):
                    yield i
        self.spinner = spinner_generator()

    def next(self, *args):
        self.out.write('\r%s' % self.spinner.next())

    def finished(self):
        self.running = False
        self.out.write('\r')


def download(url, dest=None):
    """Copy the contents of a file from a given URL
    to a local file and creates the parent dir if don't exists.
    """
    # check if the url it's ok
    try:
        urllib2.urlopen(url)
    except  urllib2.HTTPError, e:
        sys.stdout.write('\n')
        print >>sys.stderr, e, ': ', url
        sys.exit(1)
    if dest is None:
        dest = url.split('/')[-1]
    elif not os.path.exists(os.path.dirname(dest)):
        os.makedirs(os.path.dirname(dest))
    p = ProgressSpinner()
    filename, headers = urllib.urlretrieve(url, dest, reporthook=p.next)
    p.finished()

def fetch_packages():
    all = config.plugins + config.packages + config.desktop_packages + config.desktop_plugins
    for dep in all:
        fullname = [dep['name']]
        if dep.get('version', None):
            fullname.append('-' + dep['version'])
        elif dep.get('revno', None):
            fullname.append('-r' + dep['revno'])
        fullname.append('.tar.gz')
        dep_full_name = ''.join(fullname)
        tarball_path = os.path.join(external_dir, dep_full_name)
        if os.path.exists(tarball_path):
            print >> sys.stderr, dep_full_name, " already exists"
            continue
        elif dep.get('url', None):
            print "Fetching ", dep['name'],
            sys.stdout.flush()
            dest = os.path.join(external_dir, dep['url'].split('/')[-1])
            download(dep['url'], dest)
        elif dep.get('branch', None):
            print "Fetching ", dep['name']
            tree_dir = os.path.join(external_dir, dep['name'])
            if os.path.exists(tree_dir):
                subprocess.call(['bzr', 'pull', dep['branch']],
                    cwd=tree_dir)
            else:
                subprocess.call(['bzr', 'co', dep['branch'], os.path.join(external_dir, dep['name']), ])

            if check_revno(tree_dir, dep):
                subprocess.call(['bzr', 'export', tarball_path, os.path.join(external_dir, dep['name'])])

def fetch_pdf_docs():
    version = config.packages['bzr']['version']
    user_url = 'http://doc.bazaar-vcs.org/bzr.%s/downloads/pdf-en/' % version
    dev_url = 'http://doc.bazaar-vcs.org/bzr.%s/downloads/pdf-developers/' % version
    import re
    regexp = re.compile('.*href="(?P<href>.*.pdf)"')
    for url in [dev_url, user_url]:
        print "checking acces to: ", url
        f = urllib2.urlopen(url)
        hrefs = [regexp.match(line).groups()[0] for line in f.readlines() \
                    if 'href' in line and 'pdf' in line]
        f.close()
        external_dir = os.path.join(config.base, 'Documentation')
        if not os.path.exists(external_dir):
            os.makedirs(external_dir)
        for href in hrefs:
            dest = os.path.join(external_dir, doc_names[href])
            if not os.path.exists(os.path.dirname(dest)):
                os.makedirs(os.path.dirname(dest))
            p = ProgressSpinner()
            src = url+ "/" + href
            print "downloading: ", src
            filename, headers = urllib.urlretrieve(src, dest, reporthook=p.next)
            p.finished()


doc_names = {'bzr-en-architecture-overview.pdf':'Developer/Architecture overview.pdf',
'bzr-en-developer-guide.pdf':'Developer/Developer guide.pdf',
'bzr-en-integration-guide.pdf':'Developer/Integration guide.pdf',
'bzr-en-testing-guide.pdf':'Developer/Testing guide.pdf',
'bzr-en-quick-reference.pdf':'Quick reference.pdf',
'bzr-en-tutorial-centralized.pdf': 'Centralized workflow tutorial.pdf',
'bzr-en-tutorial-mini.pdf':'Bazaar in five minutes.pdf',
'bzr-en-tutorial-with-launchpad.pdf':'Using Bazaar with launchpad.pdf',
'bzr-en-tutorial.pdf':'Tutorial.pdf',
'bzr-en-upgrade-guide.pdf':'Upgrade guide.pdf',
'bzr-en-user-guide.pdf':'User guide.pdf'}


if __name__ == '__main__':
    parser = OptionParser()
    parser.add_option('-p', '--packages', dest='packages', action="store_true",
                      help='download dependencies')
    parser.add_option('-d', '--docs', dest='docs', action="store_true",
                      help='download documentation in pdf format')
    parser.add_option('-c', '--config', dest='config', action='store', default='config')
    (options, args) = parser.parse_args()

    try:
        global config
        config = __import__(options.config, globals(), locals(), [], 0)
    except ImportError:
        print "ERROR: failed to import config file"
        sys.exit(1)

    external_dir = os.path.join(config.base, 'src')

    print "using %r" % external_dir
    if options.docs:
        fetch_pdf_docs()
    elif options.packages:
        fetch_packages()
    else:
        parser.print_usage()
